// Gibt in Abhängigkeit der einzustellenden Geschwindigkeit und Höhe Warnungen aus

#include <Servo.h>               // Servo-Libary einbinden


// In den folgenden Zeilen werden die Anschlüsse definiert, an denen die externen Bauteile angeschlossen werden soll. 
// Dadurch eine Änderung des Schalplans einfach in das Programmm eingearbeitet werden.
#define GearSw 5                // Konstante, Schalter für Fahrwerk auf Pin 5
                                //  0 = Gear Up
                                //  1 = Gear Down
#define ResetSw 7               // Konstante, Akustische Warnung ausschalten auf Pin 7
#define ServoPin 9              // Konstante, Servo an Digital PWM 9
#define SpeedPin 0              // Konstante, Potie Speed an AnalogIn A0
#define AltitudePin 1           // Konstante, Potie Altitude an AnalogIn A1
#define LED1 10                 // Konstante, LED Rot Gear Unsafe an Digital 10
#define LED2 11                 // Konstante, LED Grün Gear Down+Locked an Digital 11
#define LED3 12                 // Konstante, LED Gelb Warning an Digital 12
#define Speaker 8               // Konstante, Lautsprecher an Digital 8
#define catLED 13               // Konstante, Überwachungs-LED Watchcat 

int servowert=0;                // Variable servowert: Wert/Stellung des Servos
int geschw = 0;                 // Variable für Geschwindigkeit
int hoehe = 0;                  // Variable für Höhe
int althoehe = 300;             // Variable für vorherige Höhe bei Ausgabe der Warnung
bool GearWork = LOW;            // Variable 1=Fahrwerk wird gerade gefahren                           wird die überhaupt benötigt? 
bool WarnStatus = LOW;          // Status der Warning-LED
bool ResetStatus = LOW;         // Status Reset für Abschalten der akustischen Warnung 1=Reset wurde betätigt -> Ton aus
long WarnTime = millis();       // Variable für Warnungszeitpunkt
long CatTime = millis();        // Variable für Watchcat
long DelayTime = millis();      // Variable für Verzögerung ohne delay
bool CatStatus = 1;             // Variable für CatLED (High/Low - an/aus)
byte GearStatus = 0;            // Status des Fahrwerks: 1=ausgefahren, 0=eingefahren
long GearUpTime = 0;            // Startzeitpunkt Fahrwerk ausfahren
long GearDownTime = 0;          // Startzeitpunkt Fahrwerk einfahren

Servo myservo;                   // Servo der Variable myservo zuordnen

void setup() 
  {
  Serial.begin(9600);             // Ausgabe über serialen Port ermöglichen zum Testen
  myservo.attach(ServoPin);       // Servo dem Pin ServoPin zuordnen
  pinMode(catLED, OUTPUT);        // Pin catLED als Ausgang setzen
  pinMode(LED1, OUTPUT);          // Pin LED1 als Ausgang setzen
  pinMode(LED2, OUTPUT);          // Pin LED2 als Ausgang setzen
  pinMode(LED3, OUTPUT);          // Pin LED3 als Ausgang setzen
  pinMode(Speaker, OUTPUT);       // Pin Speaker als Ausgang setzen
  pinMode(GearSw, INPUT);         // Pin ModeSw als Eingang setzen            Anmerkung:  Eingänge müssen nicht zwangsweise zugewiesen werden. 
  pinMode(ResetSw, INPUT);        // Pin UpSw als Eingang setzen                          Das wurde hier aus Gründen der Übersichtlichkeit gemacht. 
  digitalWrite(GearSw,HIGH);      // Internen Pullup für Schalter GearSw einschalten
  digitalWrite(ResetSw,HIGH);     // Internen Pullup für Schalter ResetSw einschalten
  }


void loop() 
  {
     watchcat();                  // Unterroutine Watchcat aufrufen
     
     // Analoger Wert Geschwindigkeit einlesen
     geschw = analogRead(SpeedPin); 
     // Skalieren des Sollwertes für den Servo
     // 0 - 800 entsprechen 0 - 121 kn
     // 800 da es aufgrund des Abgleichs nur 4V statt 5V sein werden 
     // Somit entspricht bei Abgleich vom Trimmer auf 100 Ohm
     // der Einstellbereich des Poties 0 bis 100 kn
     geschw = map(geschw, 0, 800, 0, 121);

     // Analoger Wert Höhe einlesen
     hoehe = analogRead(AltitudePin); 
     // Skalieren des Sollwertes für den Servo
     // 0 - 800 entsprechen 0 - 243 ft
     // 800 da es aufgrund des Abgleichs nur 4V statt 5V sein werden 
     // Somit entspricht bei Abgleich vom Trimmer auf 100 Ohm
     // der Einstellbereich des Poties 0 bis 200 ft
     hoehe = map(hoehe, 0, 800, 0, 243);
     Serial.println(geschw);
     Serial.println(hoehe);


     // Wenn ResetStatus = 1, aber die Höhe um mind. 10 ft verringert wird,
     // wird der ResetStatus wieder auf 0 gesetzt, damit die Warnung wieder hörbar ist
     if ((althoehe - hoehe) > 10 && ResetStatus ==1)
     {
      ResetStatus = 0;
      althoehe = hoehe;
     }

  // Bei Betätigung des Reset-Tasters (schaltet auf Masse!) wird der Reset-Status auf 1 gesetzt
  if (digitalRead(ResetSw) == 0)
      {
      ResetStatus = 1;
      althoehe = hoehe;   // Höhe bei Reset speichern 
      digitalWrite(Speaker, LOW); 
      }

  // Fahrwerk einfahren
  if (digitalRead(GearSw)==1 && GearStatus==1)      //GearSw auf Up?
    {
      ResetStatus = 0;  // Lautsprecher wieder freigeben
      digitalWrite (LED2, LOW);   // Gear Down&Locked ausschalten
      digitalWrite (LED1, HIGH);  // Gear Unsafe einschalten
      int i=180;
      while(i>90)
      {
        if ((millis()-DelayTime) > 30)          
        {
        DelayTime = millis();
        myservo.write(i);
        i--;
        FuncTerrainWarn();  // Muss eine Terrain-Warnung ausgegen werden? 
        FuncGearWarn();     // Muss eine Fahrwerk-Warnung ausgegebn werden? 
        }
      }
      GearStatus=0;
      GearUpTime=millis();
    }

  // Simulierte Wartezeit bis Fahrwerk eingefahren und verriegelt
  if ((millis()-GearUpTime) > 1000 && GearStatus==0 && GearUpTime > 0)
    {
      digitalWrite (LED1, LOW);  // Gear Unsafe ausschalten
      GearUpTime=0;
    }
    
  // Fahrwerk ausfahren
  if (digitalRead(GearSw)==0 && GearStatus==0)      //GearSw auf Down?
    {
      ResetStatus = 0;  // Lautsprecher wieder freigeben
      digitalWrite (LED1, HIGH);  // Gear Unsafe einschalten
      int i=90;
      while(i<180)
      {
        if ((millis()-DelayTime) > 30)          
        {
        DelayTime = millis();
        myservo.write(i);
        i++;
        FuncTerrainWarn();  // Muss eine Terrain-Warnung ausgegen werden? 
        FuncGearWarn();     // Muss eine Fahrwerk-Warnung ausgegebn werden? 
        }
      }
      GearStatus=1;
      GearDownTime=millis();
    }

  // Simulierte Wartezeit bis Fahrwerk ausgefahren und verriegelt
  if ((millis()-GearDownTime) > 1000 && GearStatus==1 && GearDownTime > 0)
    {
      digitalWrite (LED1, LOW);  // Gear Unsafe ausschalten
      digitalWrite (LED2, HIGH); // Gear down&locked einschalten
      GearDownTime=0;
    }

  FuncTerrainWarn();  // Muss eine Terrain-Warnung ausgegen werden? 
  FuncGearWarn();     // Muss eine Fahrwerk-Warnung ausgegeben werden? 
  FuncWarnOff();      // Falls keine Warnung ausgegeben werden muss alles ausschalten

}

void watchcat()
{
// Die Funktion watchcat lässt die LED an PIN 13 im ca. 1 Hz-Takt blinken.
// Dieses hat nichts mit der Funktion des Hauptprogramm zu tun, sondern
// soll nur zur Kontrolle dienen, ob der Arduino ordnungsgemäß das
// Programm arbeitet oder hängt. 
// !! Achtung: bei einem delay oder bei Schleifen im Programm blinkt die LED nicht !!
  if ((millis()-CatTime) > 500)
    {
      CatTime = millis();
      CatStatus = !CatStatus;
      digitalWrite(catLED, CatStatus);
    }
  
}

void FuncTerrainWarn()
{
  //Terrain-Warnung bei 60 kn und schneller + unter 150 ft (Fahrwerk egal)
    if (geschw >= 60 && hoehe < 150)
    {
      if ((millis() - WarnTime) > 150)
      {
      WarnTime = millis();
      WarnStatus = !WarnStatus;
      digitalWrite(LED3, WarnStatus);
      if (ResetStatus == 0)
      {
        digitalWrite(Speaker, WarnStatus); 
      }
      }
      Serial.println("Terrain-Warnung");
    }
}

void FuncGearWarn()
{
  // Fahrwerk-Warnung bei unter 60 kn + unter 150 ft + Fahrwerk eingefahren
  if (geschw < 60 && hoehe < 150 && GearStatus == 0)
    {
      if (millis()- WarnTime > 500)
      {
      WarnTime = millis();
      WarnStatus = !WarnStatus;
      digitalWrite(LED3, WarnStatus);
      if (ResetStatus == 0)
      {
        digitalWrite(Speaker, WarnStatus); 
      }
      }
      Serial.println("Fahrwerk-Warnung");
    }
} 

void FuncWarnOff()
{
  // Sobald die Höhe über 150 ft ist oder Geschwindigkeit kleiner 60
  // werden die Warnungen ausgeschaltet
  if (hoehe >= 150 || (geschw < 60 && GearStatus == 1))
  {
    digitalWrite(LED3, LOW);
    digitalWrite(Speaker, LOW); 
    ResetStatus = 0;
    Serial.println("Warnung Off");
  }
}
